@@ -65,8 +65,9 @@ class Server(IServer):
         self.desired_caps: Set[ICapability] = set([])
 
         self._read_queue:    Deque[Line] = deque()
-        self._process_queue: Deque[Line] = deque()
+        self._process_queue: Deque[Tuple[Line, Optional[Emit]]] = deque()
 
+        self._ping_sent   = False
         self._read_lguard = RLock()
         self.read_lock    = self._read_lguard
         self._read_lwork  = asyncio.Lock()
@@ -281,7 +282,6 @@ class Server(IServer):
                 self._read_queue.append(line)
 
     async def _read_lines(self):
-        ping_sent = False
         while True:
             async with self._read_lguard:
                 pass
@@ -297,13 +297,14 @@ class Server(IServer):
 
                     for done in dones:
                         if isinstance(done.result(), Line):
-                            ping_sent = False
-                            line      = done.result()
-                            self._process_queue.append(line)
+                            self._ping_sent = False
+                            line = done.result()
+                            emit = self.parse_tokens(line)
+                            self._process_queue.append((line, emit))
                         elif done.result() is None:
-                            if not ping_sent:
+                            if not self._ping_sent:
                                 await self.send(build("PING", ["hello"]))
-                                ping_sent = True
+                                self._ping_sent = True
                             else:
                                 await self.disconnect()
                                 raise ServerDisconnectedException()
@@ -311,8 +312,7 @@ class Server(IServer):
                         notdone.cancel()
 
             else:
-                line = self._process_queue.popleft()
-                emit = self.parse_tokens(line)
+                line, emit = self._process_queue.popleft()
                 await self._on_read(line, emit)
 
     async def wait_for(self,
@@ -334,7 +334,9 @@ class Server(IServer):
                     while True:
                         line = await self._read_line(timeout)
                         if line:
-                            self._process_queue.append(line)
+                            self._ping_sent = False
+                            emit = self.parse_tokens(line)
+                            self._process_queue.append((line, emit))
                             if response_obj.match(self, line):
                                 return line
 
